function [hash, status] = BLAKE256(packet, salt)
%Ova funkcija prihvata sa ulaza paket i salt, i racuna hash po blake256
%algoritmu. Za salt je ocekivan niz od cetiri 32 bitne vrednosti
%(SHA-3 proposal BLAKEstrana 8 poglavlje 2.1.2) a ako
%je niz drugaciji funkcija javlja gresku



if(size(salt,1) ~= 4)
    status = 1;
    display('Duzina salt-a je razlicita od 4 32-bitne vrednosti');
    return;
else
    status = 0;
end


%Inicijalne vrednosti preuzete iz SHA-3 proposal BLAKEstrana 8 poglavlje 2.1.1
initial_values = ['6A09E667'; 'BB67AE85'; '3C6EF372'; 'A54FF53A'; '510E527F'; '9B05688C'; '1F83D9AB'; '5BE0CD19'];
initial_values = hex2dec(initial_values);
initial_values = cast(initial_values,'uint32');

%Konstante preuzete iz SHA-3 proposal BLAKEstrana 8 poglavlje 2.1.1
constants = ['243F6A88'; '85A308D3'; '13198A2E'; '03707344'; 'A4093822'; '299F31D0'; '082EFA98'; 'EC4E6C89'; 
    '452821E6'; '38D01377'; 'BE5466CF'; '34E90C6C'; 'C0AC29B7'; 'C97C50DD'; '3F84D5B5'; 'B5470917'];
constants = hex2dec(constants);
constants = cast(constants,'uint32');

%permutaciona matrica uzeta iz SHA-3 proposal BLAKEstrana 9 poglavlje 2.1.2
permutation_matrix = zeros(10, 16, 'uint32');
permutation_matrix = [ 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15;
                       14 10 4 8 9 15 13 6 1 12 0 2 11 7 5 3;
                       11 8 12 0 5 2 15 13 10 14 3 6 7 1 9 4;
                       7 9 3 1 13 12 11 14 2 6 5 10 4 0 15 8;
                       9 0 5 7 2 4 10 15 14 1 11 12 6 8 3 13;
                       2 12 6 10 0 11 8 3 4 13 7 5 15 14 1 9;
                       12 5 1 15 14 13 4 10 0 7 6 3 9 2 8 11;
                       13 11 7 14 12 1 3 9 5 0 15 4 8 6 2 10;
                       6 15 14 9 11 3 0 8 12 2 13 7 1 4 10 5;
                       10 2 8 4 7 6 1 5 15 11 9 14 3 12 13 0];
permutation_matrix = permutation_matrix + 1;


%Duzina paketa u bitima.
packet_length = 8*size(packet,1);

%Blake algoritam zahteva sesnaest 32bitnih poruka = 512 bita
%Na osnovu odnosa duzine paketa u bitima i 512 bita mozemo da dodjemo
%do broja ulancanih blake 256 poziva. Ukoliko taj odnos nije ceo broj potrebno
%je odraditi pedovanje. U slucaju pedovanja ovaj broj treba uvecati za
%jedan ili dva sto ce se uraditi posle analize pedovanja.
num_of_blake_base = floor(packet_length/512);

%Pedovanje u nastavku je uradjeno na osnovu SHA-3 proposal BLAKE dokumenta
%strana 11 poglavlje 2.1.3

%Broj bita u poslednjem bloku. Ovaj podatak je potreban kako bi znali
%koliko bita za paddovanje je potrebno.
remainder = mod(packet_length,512);

%Minimalni broj bita koji moze da se peduje slucaj 101L(jedna jedinica
%jedna nula jedna jedinica i duzina u 64bitnom formatu)
minimum_bits_for_padding = 1 + 1 + 1 + 64;


%Ako je broj bita do prvog punog broja od 512 bita manji od 67 tj minimalnog peda onda je potrebno
%dodati jos jedan dodatni ceo blok. U nasem slucaju mi imamo granularnost
%od osam bita sto znaci da je broj bita na ulazu deljiv sa 8. Takodje i
%padding mora da bude deljiv sa 8.
if(remainder==0)
    num_of_blake = num_of_blake_base;
    num_padding_bits = 0;
elseif(512-remainder<minimum_bits_for_padding)
    num_of_blake = num_of_blake_base + 2;
    %broj bita koji se peduje. broj je sigurno deljiv sa 8 zbog prirode
    %ulaza
    num_padding_bits = 512 + 512 - remainder;
else
    num_of_blake = num_of_blake_base + 1;
    %broj bita koji se peduje. broj je sigurno deljiv sa 8 zbog prirode
    %ulaza
    num_padding_bits = 512-remainder; 
end

if(remainder~=0)
    
    pad_array = zeros(num_padding_bits/8,1,'uint8');
    if(num_padding_bits == 72)
        pad_array(1) = bin2dec('10000001');
    elseif(num_padding_bits == 80)
        pad_array(1) = bin2dec('10000000');
        pad_array(end-8) = bin2dec('00000001');
    else
        pad_array(1) = bin2dec('10000000');
        pad_array(2:end-9) = bin2dec('00000000');
        pad_array(end-8) = bin2dec('00000001');
    end;
    
  
    pad_array(end-7) = bitand(cast(packet_length,'uint64'),hex2dec('FF00000000000000'))/2^56;
    pad_array(end-6) = bitand(cast(packet_length,'uint64'),hex2dec('00FF000000000000'))/2^48;
    pad_array(end-5) = bitand(cast(packet_length,'uint64'),hex2dec('0000FF0000000000'))/2^40;
    pad_array(end-4) = bitand(cast(packet_length,'uint64'),hex2dec('000000FF00000000'))/2^32;
    pad_array(end-3) = bitand(cast(packet_length,'uint64'),hex2dec('00000000FF000000'))/2^24;
    pad_array(end-2) = bitand(cast(packet_length,'uint64'),hex2dec('0000000000FF0000'))/2^16;
    pad_array(end-1) = bitand(cast(packet_length,'uint64'),hex2dec('000000000000FF00'))/2^8;
    pad_array(end) = bitand(cast(packet_length,'uint64'),hex2dec('00000000000000FF'));
    
    packet = [packet; pad_array];
    
end


%Inicijalizacija counter promenljivih. Svaki Blake block zahteva kao ulaz
%counter vrednost koji predstavlja broj obradjenih podataka do tog
%trenutka. To cemo pokazati na primeru. Neka je potrebno obraditi 1700
%bita. Prvo cemo pedovati duzinu do 2048 bita tj dodacemo 358 bita.
%Pedovanih 2048 bita ce se obraditi u 4 chainovana black bloka. Vrednost
%countera na ulazu u prvi blok bice 512, na ulazu drugog bloka 1024, na ulazu treceg
%1536 i na ulazu u cetvrti 1700. Prilikom racunanja counter vrednosti
%padovani biti ne ulaze u obzir.
counter = zeros(num_of_blake,1,'uint64');
for ii=1:num_of_blake_base
    counter(ii) = 512*ii;
end

counter(num_of_blake_base+1) = packet_length;

if((num_of_blake-num_of_blake_base) == 2)
    counter(num_of_blake_base+2) = 0;
end


%Niz koji ce drzati pedovan niz konvertovan u 32 bitne reci
message = zeros((num_padding_bits+packet_length)/32,1,'uint32');
packet = cast(packet,'uint32');
for ii=1:size(message,1)
   message(ii) =  2^24*packet(4*(ii-1)+1) + 2^16*packet(4*(ii-1)+2) + 2^8*packet(4*(ii-1)+3) + packet(4*(ii-1)+4); 
end
%Konvertujemo matricu dtako da svaka vrsta predstavlja ulaz u odgovarajuci blake block
message = reshape(message,16,num_of_blake);
message = message';

%chain values
chain_values = initial_values; 


for ii = 1 : ((num_padding_bits+packet_length)/512) %broj poziva blake funkcija
    %Initialise
    [v_data] = blake256_initialization(chain_values,salt,constants,counter, ii);
    
    %Do rounds for chain block
    v_data = blake256_rounds(v_data,message,permutation_matrix,constants,ii);
    
    %Finalise
    [chain_values] = blake256_finalization(chain_values,v_data, salt);
    
end %endfor

%hash value iznosi 256 bita a to je 32 bajtova
hash = zeros(32,1,'uint8');

for ii=1:8
    hash(4*(ii-1)+1) = bitand(chain_values(ii),hex2dec('FF000000'))/2^24;
    hash(4*(ii-1)+2) = bitand(chain_values(ii),hex2dec('00FF0000'))/2^16;
    hash(4*(ii-1)+3) = bitand(chain_values(ii),hex2dec('0000FF00'))/2^8;
    hash(4*(ii-1)+4) = bitand(chain_values(ii),hex2dec('000000FF'));
end;




end

