function [packet, status] = ip_packet_generation(payload_length)

%Ova funkcija generise ip paket koji se sastoji iz hedera i podatka. Podaci
%se random generisu.
%Broj bajtova podatka je specifiran ulaznim parametrom funkcije. Heder
%se sastoji iz 20 bajtova. Maksimalna duzina podatka koja ova funkcija
%generise je 1448 bajtova a minimalna 1 bajt. Ukoliko je specificirana
%velicina podatka izvan ovih granica funkcija ce vratiti status razlicit od
%nule.

heder_length = 20;
total_length = heder_length + payload_length;
packet = zeros(total_length,1,'uint8');

if(payload_length<1 || payload_length>1448)
    status = 1;
    display('Duzina podatka treba biti izmedju 1 i 1448 bajtova');
    return;
else
    status = 0;
end

%prvi bajt ip hedera(verzija+ duzina hedera) u heksadecimalnom formatu
%broj 4 predstavlja IPv4 verziju protokola
%broj 5 predstavlja broj 32bitnih reci u hederu. Posto je heder duzine
%20 bajtova to je broj 32-bitnih reci jednak 5
packet(1) = hex2dec('45');
%drugi bajt tip servisa
packet(2) = hex2dec('00');
%treci i cetvrti bajt cine ukupnu duzinu paketa ukljucujuci i heder
packet(3) = bitand(total_length,hex2dec('FF00'))/2^8;  
packet(4) = bitand(total_length,hex2dec('00FF'));
%peti i sesti bajt identifikacija
packet(5) = hex2dec('00');  
packet(6) = hex2dec('00');
%sedmi i osmi bajt flegovi i fragmentacija
packet(7) = hex2dec('00');  
packet(8) = hex2dec('00');
%deveti bajt time to live
packet(9) = hex2dec('40');
%deseti bajt protokol sledeceg sloja. Vrednost '11' je za UDP.
packet(10) = hex2dec('11');
%trinaesti, cetrnaesti, petnaesti i sesnaesti bajt cine izvorisnu IP
%adresu. Setovani bajtovi cine adresu 192.168.1.100
packet(13) = hex2dec('c0');
packet(14) = hex2dec('a8');
packet(15) = hex2dec('01');
packet(16) = hex2dec('64');
%sedamnaesti, osamnaesti, devetnaesti i dvadeseti bajt cine destinacionu IP
%adresu. Setovani bajtovi cine adresu 192.168.1.101
packet(17) = hex2dec('c0');
packet(18) = hex2dec('a8');
packet(19) = hex2dec('01');
packet(20) = hex2dec('65');
%jedanaesti i dvanaesti bajt cine cek sumu.
[checksum, status] = check_sum_generation(packet(1:20));
if(status ~= 0)
    display('Greska prilikom racunjanja check sume')
    status = 2;
    return
end

packet(11) = bitand(checksum,hex2dec('0000FF00'))/2^8;
packet(12) = bitand(checksum,hex2dec('000000FF'));

%ostatak cine random podaci
packet(21:end) = floor(2^8*rand(payload_length,1));

return;

end

