LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE ieee.math_real.ALL;

ENTITY addressable_shift_register IS
    GENERIC(
        DATA_SIZE : INTEGER := 20;
        MAX_DELAY : INTEGER := 20;  
        ADDR_LENGTH : INTEGER := 5      
    );
    PORT (
        clk: IN STD_LOGIC;
        --ulazni interfejs
        data_in: IN STD_LOGIC_VECTOR(DATA_SIZE-1 DOWNTO 0);
        address: IN STD_LOGIC_VECTOR(ADDR_LENGTH-1 DOWNTO 0);
        --izlazni interfejs
        data_out: OUT STD_LOGIC_VECTOR(DATA_SIZE-1 DOWNTO 0)

    );
END addressable_shift_register;

ARCHITECTURE RTL OF addressable_shift_register IS

TYPE data_buffer_type IS ARRAY (MAX_DELAY-1 DOWNTO 0) of  STD_LOGIC_VECTOR(DATA_SIZE-1 DOWNTO 0); 
SIGNAL data_buffer: data_buffer_type; 


BEGIN

register_generation : FOR N IN 0 TO MAX_DELAY-2 GENERATE

PROCESS(clk)
BEGIN
    IF (clk'EVENT AND clk = '1') THEN
        data_buffer(N+1) <= data_buffer(N);
    END IF;
END PROCESS;

END GENERATE register_generation;

PROCESS(clk)
BEGIN
    IF (clk'EVENT AND clk = '1') THEN
        data_buffer(0) <= data_in;
    END IF;
END PROCESS;

data_out <= data_buffer(conv_integer(address));

END RTL;