LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE WORK.blake_pak.ALL;

ENTITY blake_pipe IS
    PORT (
        clk: IN STD_LOGIC; 

        h: IN  STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
        m: IN  STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
        s: IN  STD_LOGIC_VECTOR (WIDTHx4-1 DOWNTO 0);
        t: IN  STD_LOGIC_VECTOR (WIDTHx2-1 DOWNTO 0);
        inEN: IN  STD_LOGIC;

        hash: OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
        outEN: OUT  STD_LOGIC
    );
END blake_pipe;

ARCHITECTURE blake_pipe_a OF blake_pipe IS


    COMPONENT blake_init IS
        PORT (
            clk_in: IN STD_LOGIC; 

            h_in: IN  STD_LOGIC_WIDTHx8;
            m_in: IN  STD_LOGIC_WIDTHx16;
            s_in: IN  STD_LOGIC_WIDTHx4;
            t_in: IN  STD_LOGIC_WIDTHx2;

            h_out: OUT STD_LOGIC_WIDTHx8;
            m_out: OUT STD_LOGIC_WIDTHx16;
            s_out: OUT STD_LOGIC_WIDTHx4;
            v_out: OUT STD_LOGIC_WIDTHx16
        );
    END COMPONENT;

    COMPONENT round_pipe IS
        PORT (
            clk_in: IN STD_LOGIC; 

            round_cnt_in: IN  INTEGER;

            h_in: IN STD_LOGIC_WIDTHx8;
            m_in: IN STD_LOGIC_WIDTHx16;
            s_in: IN STD_LOGIC_WIDTHx4;
            v_in: IN STD_LOGIC_WIDTHx16;

            h_out: OUT STD_LOGIC_WIDTHx8;
            m_out: OUT STD_LOGIC_WIDTHx16;
            s_out: OUT STD_LOGIC_WIDTHx4;
            v_out: OUT STD_LOGIC_WIDTHx16
        );
    END COMPONENT;

    COMPONENT blake_fin IS
        PORT (
            clk_in: IN STD_LOGIC; 

            v_in: IN  STD_LOGIC_WIDTHx16;
            h_in: IN  STD_LOGIC_WIDTHx8;
            s_in: IN  STD_LOGIC_WIDTHx4;

            hash_out: OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0)
        );
    END COMPONENT;


TYPE h_i0 IS ARRAY (0 TO Num_of_rounds) OF STD_LOGIC_WIDTHx8;
TYPE m_i0 IS ARRAY (0 TO Num_of_rounds) OF STD_LOGIC_WIDTHx16;
TYPE s_i0 IS ARRAY (0 TO Num_of_rounds) OF STD_LOGIC_WIDTHx4;
TYPE v_i0 IS ARRAY (0 TO Num_of_rounds) OF STD_LOGIC_WIDTHx16;

SIGNAL h_in_out: h_i0;
SIGNAL m_in_out: m_i0;
SIGNAL s_in_out: s_i0;
SIGNAL v_in_out: v_i0;

SIGNAL h_array: STD_LOGIC_WIDTHx8;
SIGNAL m_array: STD_LOGIC_WIDTHx16;
SIGNAL s_array: STD_LOGIC_WIDTHx4;
SIGNAL t_array: STD_LOGIC_WIDTHx2;

SIGNAL enable_SHR_reg:  STD_LOGIC_VECTOR ( Num_of_stages-1 DOWNTO 0) := (others => '0'); 

BEGIN 
-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------
h_array(7) <= h(WIDTHx1-1 DOWNTO 0);--konverzija iz velikog vektora u niz manjih
h_array(6) <= h(WIDTHx2-1 DOWNTO WIDTHx1); --!!!!!!!!!! BITNA STVAR REDOSLED DODELJIVANJA IZ RECI U NIZ!!! OBRNUTO OD OCEKIVANOG,RAZLOG PATERNI IZ LIRERATURE
h_array(5) <= h(WIDTHx3-1 DOWNTO WIDTHx2);
h_array(4) <= h(WIDTHx4-1 DOWNTO WIDTHx3);
h_array(3) <= h(WIDTHx5-1 DOWNTO WIDTHx4); 
h_array(2) <= h(WIDTHx6-1 DOWNTO WIDTHx5); 
h_array(1) <= h(WIDTHx7-1 DOWNTO WIDTHx6); 
h_array(0) <= h(WIDTHx8-1 DOWNTO WIDTHx7); 

m_array(15) <= m(WIDTHx1-1 DOWNTO 0);
m_array(14) <= m(WIDTHx2-1 DOWNTO WIDTHx1);  
m_array(13) <= m(WIDTHx3-1 DOWNTO WIDTHx2);  
m_array(12) <= m(WIDTHx4-1 DOWNTO WIDTHx3); 
m_array(11) <= m(WIDTHx5-1 DOWNTO WIDTHx4);  
m_array(10) <= m(WIDTHx6-1 DOWNTO WIDTHx5); 
m_array(9)  <= m(WIDTHx7-1 DOWNTO WIDTHx6); 
m_array(8)  <= m(WIDTHx8-1 DOWNTO WIDTHx7);  
m_array(7)  <= m(WIDTHx9-1 DOWNTO WIDTHx8);  
m_array(6)  <= m(WIDTHx10-1 DOWNTO WIDTHx9);  
m_array(5)  <= m(WIDTHx11-1 DOWNTO WIDTHx10);  
m_array(4)  <= m(WIDTHx12-1 DOWNTO WIDTHx11); 
m_array(3)  <= m(WIDTHx13-1 DOWNTO WIDTHx12); 
m_array(2)  <= m(WIDTHx14-1 DOWNTO WIDTHx13);  
m_array(1)  <= m(WIDTHx15-1 DOWNTO WIDTHx14);  
m_array(0)  <= m(WIDTHx16-1 DOWNTO WIDTHx15);  

s_array(3)  <= s(WIDTHx1-1 DOWNTO 0);
s_array(2)  <= s(WIDTHx2-1 DOWNTO WIDTHx1);  
s_array(1)  <= s(WIDTHx3-1 DOWNTO WIDTHx2);  
s_array(0)  <= s(WIDTHx4-1 DOWNTO WIDTHx3); 

t_array(1)  <= t(WIDTHx1-1 DOWNTO 0);
t_array(0)  <= t(WIDTHx2-1 DOWNTO WIDTHx1);  
-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------

    blake_init_com: blake_init 
    PORT MAP (
        clk_in => clk,

        h_in => h_array,
        m_in => m_array,
        s_in => s_array,
        t_in => t_array,

        h_out => h_in_out(0),
        m_out => m_in_out(0),
        s_out => s_in_out(0),
        v_out => v_in_out(0) 
    );


    Round_Gen:
    FOR i IN 0 TO Num_of_rounds-1 GENERATE
        round_pipe_com: round_pipe 
        PORT MAP (
            clk_in => clk ,
            round_cnt_in => i ,

            h_in => h_in_out(i) ,
            m_in => m_in_out(i) ,
            s_in => s_in_out(i) ,
            v_in => v_in_out(i) ,

            h_out => h_in_out(i+1) ,
            m_out => m_in_out(i+1) ,
            s_out => s_in_out(i+1) ,
            v_out => v_in_out(i+1) 
        );
    END GENERATE;

    blake_fin_com: blake_fin 
    PORT MAP (
        clk_in => clk ,

        v_in => v_in_out(Num_of_rounds) ,
        h_in => h_in_out(Num_of_rounds) ,
        s_in => s_in_out(Num_of_rounds) ,

        hash_out => hash 
    );

-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------

    PROCESS(clk)
    BEGIN
        IF (clk'EVENT AND clk = '1') THEN
            enable_SHR_reg(Num_of_stages-2 DOWNTO 0) <= enable_SHR_reg(Num_of_stages-1 DOWNTO 1);
            enable_SHR_reg(Num_of_stages-1) <= inEN;    
        END IF;
    END PROCESS;

    outEN <= enable_SHR_reg(0);

-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------

END ARCHITECTURE;