LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE WORK.blake_pak.ALL;

ENTITY round_pipe IS
    PORT (
        clk_in : IN STD_LOGIC; 

        round_cnt_in : IN  INTEGER;

        h_in : IN STD_LOGIC_WIDTHx8;
        m_in : IN STD_LOGIC_WIDTHx16;
        s_in : IN STD_LOGIC_WIDTHx4;
        v_in : IN STD_LOGIC_WIDTHx16;

        h_out : OUT STD_LOGIC_WIDTHx8;
        m_out : OUT STD_LOGIC_WIDTHx16;
        s_out : OUT STD_LOGIC_WIDTHx4;
        v_out : OUT STD_LOGIC_WIDTHx16
    );
END round_pipe;

ARCHITECTURE round_pipe_a OF round_pipe IS

    COMPONENT round_1_pipe IS
        PORT (
            clk_in : IN STD_LOGIC; 

            round_cnt_in : IN  INTEGER;

            round_1_h_in : IN STD_LOGIC_WIDTHx8;
            round_1_m_in : IN STD_LOGIC_WIDTHx16;
            round_1_s_in : IN STD_LOGIC_WIDTHx4;
            round_1_v_in : IN STD_LOGIC_WIDTHx16;

            round_1_h_out : OUT STD_LOGIC_WIDTHx8;
            round_1_m_out : OUT STD_LOGIC_WIDTHx16;
            round_1_s_out : OUT STD_LOGIC_WIDTHx4;
            round_1_v_out : OUT STD_LOGIC_WIDTHx16
        );
    END COMPONENT;

    COMPONENT round_2_pipe IS
        PORT (
            clk_in : IN STD_LOGIC; 

            round_cnt_in : IN  INTEGER;

            round_2_h_in : IN STD_LOGIC_WIDTHx8;
            round_2_m_in : IN STD_LOGIC_WIDTHx16;
            round_2_s_in : IN STD_LOGIC_WIDTHx4;	
            round_2_v_in : IN STD_LOGIC_WIDTHx16;

            round_2_h_out : OUT STD_LOGIC_WIDTHx8;
            round_2_m_out : OUT STD_LOGIC_WIDTHx16;
            round_2_s_out : OUT STD_LOGIC_WIDTHx4;	
            round_2_v_out : OUT STD_LOGIC_WIDTHx16
        );
    END COMPONENT;

    SIGNAL round_h: STD_LOGIC_WIDTHx8;
    SIGNAL round_m: STD_LOGIC_WIDTHx16;
    SIGNAL round_s: STD_LOGIC_WIDTHx4;
    SIGNAL round_v: STD_LOGIC_WIDTHx16;

BEGIN 

    round_1_pipe_com: round_1_pipe 
    PORT MAP (
        clk_in => clk_in ,

        round_cnt_in => round_cnt_in ,

        round_1_h_in => h_in ,
        round_1_m_in => m_in ,
        round_1_s_in => s_in ,
        round_1_v_in => v_in ,

        round_1_h_out => round_h ,
        round_1_m_out => round_m ,
        round_1_s_out => round_s ,
        round_1_v_out => round_v 
    );


    round_2_pipe_com: round_2_pipe 
    PORT MAP (
        clk_in => clk_in ,

        round_cnt_in => round_cnt_in ,

        round_2_h_in => round_h ,
        round_2_m_in => round_m ,
        round_2_s_in => round_s ,
        round_2_v_in => round_v ,

        round_2_h_out => h_out ,
        round_2_m_out => m_out ,
        round_2_s_out => s_out ,
        round_2_v_out => v_out 
    );

END ARCHITECTURE;