-------------------------------------------------------------------------------
-- File         : BLAKE_PIPE_ROUND_1.vhd
-- Autor        : Goran Ognjanovic
-- Simulator    : ModelSim
-- Sintetizator : Quartus II
-- Datum        : 2012, Avgust 9th
-- Opis modula  : Ovaj modul predstavlja prvu polovinu Round funkcije! 
--                Sastoji se od 4 bloka G fukcija
--                G0(v0,v4,v8,v12)
--                G2(v1,v5,v9,v13)
--                G3(v2,v6,v10,v14)
--                G4(v3,v7,v11,v15)
--                Zbog pipeline protocne obrade, moramo da prosledjujemo i registrujemo
-- Modifikovala : Danica Golubicic          
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.blake_pak.ALL;

ENTITY round_1_pipe IS
    PORT (
       clk_in : IN STD_LOGIC; 

       round_cnt_in : IN  INTEGER;

       round_1_h_in : IN STD_LOGIC_WIDTHx8;
       round_1_m_in : IN STD_LOGIC_WIDTHx16;
       round_1_s_in : IN STD_LOGIC_WIDTHx4;
       round_1_v_in : IN STD_LOGIC_WIDTHx16;

       round_1_h_out : OUT STD_LOGIC_WIDTHx8;
       round_1_m_out : OUT STD_LOGIC_WIDTHx16;
       round_1_s_out : OUT STD_LOGIC_WIDTHx4;
       round_1_v_out : OUT STD_LOGIC_WIDTHx16
    );
END round_1_pipe;

ARCHITECTURE round_1_pipe_a OF round_1_pipe IS
    SIGNAL h_reg , h_next : STD_LOGIC_WIDTHx8 := INIT_WIDTHx8;
    SIGNAL m_reg , m_next : STD_LOGIC_WIDTHx16 := INIT_WIDTHx16;          
    SIGNAL s_reg , s_next : STD_LOGIC_WIDTHx4 := INIT_WIDTHx4;                
    SIGNAL v_reg , v_next : STD_LOGIC_WIDTHx16 := INIT_WIDTHx16;
    SIGNAL m , c :STD_LOGIC_WIDTHx8 := INIT_WIDTHx8;

    COMPONENT blake_g IS
        PORT (
            clk : IN STD_LOGIC;
            a : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            b : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            c : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            d : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);

            m0_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            m1_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            con0_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            con1_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);

            a2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            b2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            c2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
            d2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0)
        );
    END COMPONENT;

BEGIN 
    PROCESS(clk_in)
    BEGIN
        IF (clk_in'EVENT AND clk_in = '1') THEN
            h_next <= round_1_h_in;
            m_next <= round_1_m_in;
            s_next <= round_1_s_in;
            h_reg <= h_next;
            m_reg <= m_next;
            s_reg <= s_next;
            v_reg <= v_next;
        END IF;
    END PROCESS;
    
    m(0)<= round_1_m_in(m_num(round_cnt_in)(0));
    m(1)<= round_1_m_in(m_num(round_cnt_in)(1));
    m(2)<= round_1_m_in(m_num(round_cnt_in)(2));
    m(3)<= round_1_m_in(m_num(round_cnt_in)(3));
    m(4)<= round_1_m_in(m_num(round_cnt_in)(4));
    m(5)<= round_1_m_in(m_num(round_cnt_in)(5));
    m(6)<= round_1_m_in(m_num(round_cnt_in)(6));
    m(7)<= round_1_m_in(m_num(round_cnt_in)(7));
    
    c(0)<= c_con(round_cnt_in)(0);
    c(1)<= c_con(round_cnt_in)(1);
    c(2)<= c_con(round_cnt_in)(2);
    c(3)<= c_con(round_cnt_in)(3);
    c(4)<= c_con(round_cnt_in)(4);
    c(5)<= c_con(round_cnt_in)(5);
    c(6)<= c_con(round_cnt_in)(6);
    c(7)<= c_con(round_cnt_in)(7);
    
    blake_g_com_1: blake_g 
    PORT MAP (
            clk => clk_in,
            a => round_1_v_in(0) ,
            b => round_1_v_in(4) ,
            c => round_1_v_in(8) ,
            d => round_1_v_in(12) ,
            
            m0_in => m(0) ,
            m1_in => m(1) ,
            con0_in => c(0) ,
            con1_in => c(1) ,

            a2 =>  v_next(0),
            b2 =>  v_next(4),
            c2 =>  v_next(8),
            d2 =>  v_next(12)
    );

    blake_g_com_2: blake_g 
    PORT MAP (
            clk => clk_in,
            a => round_1_v_in(1) ,
            b => round_1_v_in(5) ,
            c => round_1_v_in(9) ,
            d => round_1_v_in(13) ,
            
            m0_in => m(2) ,
            m1_in => m(3) ,
            con0_in => c(2) ,
            con1_in => c(3) ,

            a2 =>  v_next(1),
            b2 =>  v_next(5),
            c2 =>  v_next(9),
            d2 =>  v_next(13)
    );    

    blake_g_com_3: blake_g 
    PORT MAP (
            clk => clk_in,
            a => round_1_v_in(2) ,
            b => round_1_v_in(6) ,
            c => round_1_v_in(10) ,
            d => round_1_v_in(14) ,
            
            m0_in => m(4) ,
            m1_in => m(5) ,
            con0_in => c(4) ,
            con1_in => c(5) ,

            a2 =>  v_next(2),
            b2 =>  v_next(6),
            c2 =>  v_next(10),
            d2 =>  v_next(14)
    );


    blake_g_com_4: blake_g 
    PORT MAP (
            clk => clk_in,
            a => round_1_v_in(3) ,
            b => round_1_v_in(7) ,
            c => round_1_v_in(11) ,
            d => round_1_v_in(15) ,
            
            m0_in => m(6) ,
            m1_in => m(7) ,
            con0_in => c(6) ,
            con1_in => c(7) ,

            a2 =>  v_next(3),
            b2 =>  v_next(7),
            c2 =>  v_next(11),
            d2 =>  v_next(15)
    );

    round_1_h_out <= h_reg;
    round_1_m_out <= m_reg;
    round_1_s_out <= s_reg;
    round_1_v_out <= v_reg;
    
END ARCHITECTURE;