library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity FIFO is
    Port ( clk   : in  STD_LOGIC;
           wr    : in  STD_LOGIC;
           din   : in  STD_LOGIC_VECTOR(7 downto 0);
           empty : out STD_LOGIC;
           full  : out STD_LOGIC;
           rd    : in  STD_LOGIC;
           dout  : out STD_LOGIC_VECTOR(7 downto 0));
end FIFO;

architecture Behavioral of FIFO is
   --signal koji kaze da li je FIFO napunjen do kraja. Ukoliko jeste treba spreciti upis.
   signal i_full : std_logic;
   --signal koji kaze da li je FIFO prazan. Ako jeste treba spreciti citanje.
   signal i_empty : std_logic;
   
   --memorija za fifo
   type mem_array is array(31 downto 0) of std_logic_vector(7 downto 0);
   signal memory : mem_array;
   --adresa za upis
   signal wr_ptr : unsigned(4 downto 0) := (others => '0');
   --adresa za citanje
   signal rd_ptr : unsigned(4 downto 0) := (others => '0');
   
begin
    full  <= i_full;
    empty <= i_empty;

flag_proc: process(wr_ptr, rd_ptr)
    begin
        --ukoliko su adrese za upis i adresa za citanje iste FIFO je prazan. Na pocetku pre bilo kakve akcije imamo taj slucaj.
        if wr_ptr = rd_ptr then
            i_empty <= '1';
        else
            i_empty <= '0';
        end if;

        --ukoliko je sledeca adresa za upis jednaka adresi za citanje fifo je pun
        if wr_ptr+1 = rd_ptr then
            i_full <= '1';
        else
            i_full <= '0';
        end if;
    end process;

clk_proc: process(clk)
    begin
        if rising_edge(clk) then
            if rd = '1' then -- zahtev za citanje je poslat
                if wr = '1' then -- istovremeno i zahtev za upis
                    if i_empty = '0' then -- ako FIFO nije prazan procitaj vrednost
                        dout <= memory(to_integer(rd_ptr));
                        rd_ptr <= rd_ptr + 1;
                    end if;
                    
                    memory(to_integer(wr_ptr)) <= din; --upisi vrednost u svakom slucaju
                    wr_ptr <= wr_ptr + 1;
                    
                elsif i_empty = '0' then -- slucaj kada je poslat samo zahtev za citanje ako nije prazan procitaj
                    dout <= memory(to_integer(rd_ptr));
                    rd_ptr <= rd_ptr + 1;
                end if;
            elsif wr = '1' then --zahtev za upis je poslat bez zahteva za citanje. ako nije pun procitaj
                if i_full = '0' then
                    memory(to_integer(wr_ptr)) <= din;
                    wr_ptr <= wr_ptr + 1;
                end if;           
            end if;
        end if;
    end process;
end Behavioral;