LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;


ENTITY ip_frame_storing IS
    PORT (
        clk: IN STD_LOGIC;
        --ulazni interfejs
        start_in: IN STD_LOGIC;
        data_vld_in: IN STD_LOGIC;
        data_in: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
        total_new_length : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
        cnt: IN STD_LOGIC_VECTOR(10 DOWNTO 0);
        pading_length: IN STD_LOGIC_VECTOR(6 DOWNTO 0);
        --izlazni interfejs
        start_out: OUT STD_LOGIC;
        output_enable: OUT STD_LOGIC;
        data_out: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
    );
END ip_frame_storing;


ARCHITECTURE RTL OF ip_frame_storing IS

    COMPONENT FIFO IS
        PORT ( 
            clk   : IN  STD_LOGIC;
            wr    : IN  STD_LOGIC;
            din   : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
            empty : OUT STD_LOGIC;
            full  : OUT STD_LOGIC;
            rd    : IN  STD_LOGIC;
            dout  : OUT STD_LOGIC_VECTOR(7 DOWNTO 0));
    END COMPONENT;
    
    COMPONENT addressable_shift_register IS
    GENERIC(
        DATA_SIZE : INTEGER := 20;
        MAX_DELAY : INTEGER := 20;  
        ADDR_LENGTH : INTEGER := 5      
    );
    PORT (
        clk: IN STD_LOGIC;
        data_in: IN STD_LOGIC_VECTOR(DATA_SIZE-1 DOWNTO 0);
        address: IN STD_LOGIC_VECTOR(ADDR_LENGTH-1 DOWNTO 0);
        data_out: OUT STD_LOGIC_VECTOR(DATA_SIZE-1 DOWNTO 0)

    );
    END COMPONENT;

    CONSTANT state_MSB                : STD_LOGIC := '0';
    CONSTANT state_LSB                : STD_LOGIC := '1';
    SIGNAL state_of_receiving_byte    : STD_LOGIC := state_MSB;
    SIGNAL latched_msb_data           : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');

    SIGNAL check_sum_aux_1            : STD_LOGIC_VECTOR(16 DOWNTO 0) := (OTHERS => '0');
    SIGNAL check_sum_aux_2            : STD_LOGIC_VECTOR(15 DOWNTO 0) := (OTHERS => '0');
    SIGNAL check_sum                  : STD_LOGIC_VECTOR(15 DOWNTO 0) := (OTHERS => '0');
    
    SIGNAL byte_for_check_sum         : STD_LOGIC_VECTOR(7 downto 0)  := (OTHERS => '0');
    SIGNAL data_vld_in_d              : STD_LOGIC := '0';

    
    SIGNAL latched_check_sum          : STD_LOGIC_VECTOR(15 DOWNTO 0) := (OTHERS => '0');
    SIGNAL latch_check_sum            : STD_LOGIC := '0';
    SIGNAL latch_check_sum_d, latch_check_sum_2d : STD_LOGIC := '0';
    SIGNAL empty_fifo                 : STD_LOGIC := '0';
    SIGNAL read_fifo, read_fifo_d     : STD_LOGIC := '0';
    SIGNAL enable_fifo_reading        : STD_LOGIC := '0';
    
    SIGNAL cnt_fifo_read              : STD_LOGIC_VECTOR(10 DOWNTO 0) := (OTHERS => '0');
    SIGNAL fifo_out                   : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');


    SIGNAL address_shift_reg          : STD_LOGIC_VECTOR(6 DOWNTO 0) := (OTHERS => '0');
    SIGNAL address_shift_reg_aux      : STD_LOGIC_VECTOR(6 DOWNTO 0) := (OTHERS => '0');

    SIGNAL start_out_int              : STD_LOGIC := '0';
    SIGNAL start_out_int_aux          : STD_LOGIC_VECTOR(0 DOWNTO 0) := (OTHERS => '0');
    SIGNAL start_out_aux              : STD_LOGIC_VECTOR(0 DOWNTO 0) := (OTHERS => '0');
    
    SIGNAL output_enable_int          : STD_LOGIC:= '0';
    SIGNAL output_enable_int_aux      : STD_LOGIC_VECTOR(0 DOWNTO 0) := (OTHERS => '0');
    SIGNAL output_enable_aux          : STD_LOGIC_VECTOR(0 DOWNTO 0) := (OTHERS => '0');    
    

    SIGNAL data_out_int               : STD_LOGIC_VECTOR(7 DOWNTO 0):= (OTHERS => '0');
    
    SIGNAL pading_length_int          : STD_LOGIC_VECTOR(6 DOWNTO 0):= (OTHERS => '0');
    

BEGIN

    --Prilikom racunanja checksume izlaznog ip paketa koriste se bajtovi ulaznog ip hedera osim azuriranih bajtova za novu duzinu ip paketa. Umesto bajtova koji
    --cine checksumu ulaznog paketa podmecu se nule.
    
    

    --Proces koji odredjuje koji je sledeci bajt za racunanje checksume. U kalkulaciju nove checksume ulazi nova duzina poruke
    --a na mestu bajtova koji predstavljaju checksumu se ubacuju nule.
    PROCESS (clk)
    BEGIN
        IF (clk'EVENT AND clk = '1') THEN
            data_vld_in_d <= data_vld_in;

            IF(data_vld_in = '1' and cnt =  conv_std_logic_vector(2,11)) THEN--u masinu za kalkulaciju checksume se ubacuje nova ukupna duzina poruke (MSB bajt)
                byte_for_check_sum <= total_new_length(15 DOWNTO 8);
            ELSIF(data_vld_in = '1' and cnt =  conv_std_logic_vector(3,11)) THEN--u masinu za kalkulaciju checksume se ubacuje nova ukupna duzina poruke (LSB bajt)
                byte_for_check_sum <= total_new_length(7 DOWNTO 0);           
            ELSIF(data_vld_in = '1' and cnt =  conv_std_logic_vector(10,11)) THEN--na mestu bajtova check sume se ubacuju nule (MSB bajt)
                byte_for_check_sum <= (OTHERS => '0');  
            ELSIF(data_vld_in = '1' and cnt =  conv_std_logic_vector(11,11)) THEN--na mestu bajtova check sume se ubacuju nule (LSB bajt)
                byte_for_check_sum <= (OTHERS => '0');                  
            ELSE                                          --ostali bajtovi koji ulaze u masinu za racunanje checksume su identicni kao u ulaznoj poruci
                byte_for_check_sum <= data_in;
            END IF;
            
        END IF;
    END PROCESS;    
    


    --Masina za kreiranje checksume izlaznog paketa
    PROCESS(clk)
    BEGIN
        IF (clk'EVENT AND clk = '1') THEN
            check_sum_aux_2 <= check_sum_aux_1(15 DOWNTO 0) + check_sum_aux_1(16);

            IF(start_in = '1') THEN
                state_of_receiving_byte <= state_MSB;
                check_sum_aux_1<= (OTHERS => '0');
            ELSIF(data_vld_in_d = '1') THEN
                CASE state_of_receiving_byte IS
                    WHEN state_MSB =>
                        state_of_receiving_byte <= state_LSB;
                        latched_msb_data <= byte_for_check_sum;
                    WHEN state_LSB =>
                        state_of_receiving_byte <= state_MSB;
                        check_sum_aux_1 <= ('0' & check_sum_aux_2) + ('0'  &  latched_msb_data  &  byte_for_check_sum);
                    WHEN OTHERS =>
                END CASE;
            END IF;

        END IF;
    END PROCESS;

    check_sum <= not check_sum_aux_2;
    
    --ovaj proces ima zadatak da u odgovarajucem trenutku sacuva checksumu
    PROCESS(clk)
    BEGIN
        IF (clk'EVENT AND clk = '1') THEN
            IF (data_vld_in = '1' AND cnt = conv_std_logic_vector(19,11)) THEN
                latch_check_sum <= '1';
            ELSE
                latch_check_sum <= '0';            
            END IF;
            latch_check_sum_d  <= latch_check_sum;
            latch_check_sum_2d <= latch_check_sum_d;
            
            IF(latch_check_sum_2d = '1') THEN
               latched_check_sum <= check_sum;
            END IF;
        END IF;
    END PROCESS;

    PROCESS(clk)
    BEGIN
       IF (clk'EVENT AND clk = '1') THEN
           --FIFO se pocinje citati tek kada svi bajtovi ip hedera stignu
           IF (data_vld_in = '1' AND cnt = conv_std_logic_vector(0,11)) THEN
               enable_fifo_reading <= '0';
           ELSIF(data_vld_in = '1' AND cnt = conv_std_logic_vector(19,11)) THEN
               enable_fifo_reading <= '1';            
           END IF;

           read_fifo_d <= read_fifo;
           output_enable_int <= read_fifo_d;

           --brojac koji kontrolise koliko bajtova iz FIFO-a se procitalo.
           IF(enable_fifo_reading = '0') THEN
               cnt_fifo_read <= (OTHERS => '0');
           ELSIF(read_fifo = '1') THEN
               cnt_fifo_read <= cnt_fifo_read + conv_std_logic_vector(1,11); 
           END IF;
           
           --generisanje start signala prilikom prvog procitanog bajta ip paketa
           IF(cnt_fifo_read = conv_std_logic_vector(1,11) and read_fifo_d = '1') THEN
               start_out_int <= '1';
           ELSE
               start_out_int <= '0';
           END IF;
           
           --prilikom citanja podataka iz fifoa na mestu check sume zameniti bajtove sa dobijenom checksumom
           IF(cnt_fifo_read = conv_std_logic_vector(11,11)) THEN
               data_out_int <= latched_check_sum(15 DOWNTO 8);
           ELSIF(cnt_fifo_read = conv_std_logic_vector(12,11)) THEN
               data_out_int <= latched_check_sum(7 DOWNTO 0);
           ELSE
               data_out_int <= fifo_out;
           END IF;
           
       END IF;
    END PROCESS;
    
    
    --signal za citanje fifoa. Fifo se cita u slucaju da je citanje dozvoljeno i fifo nije prazan.
    read_fifo <= enable_fifo_reading and (not empty_fifo);
    
    
    inst_FIFO: FIFO
        PORT MAP( 
            clk => clk,
            wr => data_vld_in_d,
            din => byte_for_check_sum,
            empty => empty_fifo,
            full => open,
            rd => read_fifo,
            dout => fifo_out
    );
    
    --Podaci i valid signal iz FIFO-a moraju da se zakasne. Njihovo kasnjenje zavisi od broja padding bajtova.
    --Ukoliko nema padding-a kasnjenje je 39. Za svaki bajt padding-a kasnjenje se uvecava za jedan. Maksimalno kasnjenje je 91.
    --U tu svrhu se koristi adresni sift registar.
 

    pading_length_int <= pading_length;
    
    PROCESS (clk)
    BEGIN
        IF (clk'EVENT AND clk = '1') THEN

            address_shift_reg_aux <= pading_length_int + conv_std_logic_vector(38,7);
            
            IF(address_shift_reg_aux > conv_std_logic_vector(90,7)) THEN
                address_shift_reg <= conv_std_logic_vector(90,7);
            ELSE
                address_shift_reg <= address_shift_reg_aux;
            END IF;
            
            
        END IF;
    END PROCESS;       
    
    
    start_out_int_aux(0) <= start_out_int;

    inst_start_out_delay: addressable_shift_register
    GENERIC MAP(
        DATA_SIZE => 1,
        MAX_DELAY => 91, 
        ADDR_LENGTH => 7
    )
    PORT MAP(
        clk => clk,
        data_in => start_out_int_aux,
        address => address_shift_reg,
        data_out => start_out_aux

    );
    
    start_out <= start_out_aux(0);
    
    
    output_enable_int_aux(0) <= output_enable_int;

    inst_output_enable_delay: addressable_shift_register
    GENERIC MAP(
        DATA_SIZE => 1,
        MAX_DELAY => 91, 
        ADDR_LENGTH => 7
    )
    PORT MAP(
        clk => clk,
        data_in => output_enable_int_aux,
        address => address_shift_reg,
        data_out => output_enable_aux

    );
    
    output_enable <= output_enable_aux(0);    
    
    inst_data_out_delay: addressable_shift_register
    GENERIC MAP(
        DATA_SIZE => 8,
        MAX_DELAY => 91, 
        ADDR_LENGTH => 7
    )
    PORT MAP(
        clk => clk,
        data_in => data_out_int,
        address => address_shift_reg,
        data_out => data_out

    );    
    




END RTL;