LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;

LIBRARY std;
USE std.textio.ALL;


--Entity testbench-a je prazan
ENTITY tb_ip_frame_signature IS
END tb_ip_frame_signature;

----------------------------------------------------------------------------------------    
ARCHITECTURE tb_ip_frame_signature_a OF tb_ip_frame_signature IS
    --deklaracija komponente koja se testira
    COMPONENT ip_frame_signature IS
    PORT (
        clk: IN STD_LOGIC;
        --ulazni interfejs
        start_in: IN STD_LOGIC;
        data_vld_in: IN STD_LOGIC;
        data_in: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
        ready_to_accept: OUT STD_LOGIC;
        --izlazni interfejs
        start_out: OUT STD_LOGIC;
        output_enable: OUT STD_LOGIC;
        data_out: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
    );
    END COMPONENT;

    --Deklaracija internih signala 
    
    --Clock od 125 MHz. Kod gigabitnog eterneta se koristi ovaj clock zajedno sa osmobitnom magistralom za podatke.
    SIGNAL clk_125:  STD_LOGIC := '0'; 
    CONSTANT clk_125MHz_period : TIME := 8 ns;

    --Ulazi
    SIGNAL start_in: STD_LOGIC := '0';
    SIGNAL data_vld_in: STD_LOGIC := '0';
    SIGNAL data_in: STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');
    --Izlazi
    SIGNAL start_out: STD_LOGIC;
    SIGNAL output_enable: STD_LOGIC;
    SIGNAL data_out: STD_LOGIC_VECTOR(7 DOWNTO 0);
    SIGNAL ready_to_accept: STD_LOGIC;
--****************************************************************************************************  
    
BEGIN 
    --instanca komponente koja se testira
    uut: ip_frame_signature
    PORT MAP (
        clk => clk_125 ,
        start_in => start_in,
        data_vld_in => data_vld_in,
        data_in => data_in,
        ready_to_accept => ready_to_accept,
        start_out => start_out,
        output_enable => output_enable,
        data_out => data_out
    );

   --****************************************************************************************************
   --generator takta
   --****************************************************************************************************
    clk_125_process :PROCESS
    BEGIN
        clk_125 <= '0';
        WAIT FOR clk_125MHz_period/2;
        clk_125 <= '1';
        WAIT FOR clk_125MHz_period/2;
    END PROCESS;
--****************************************************************************************************
    --Proces koji definise test vektore. Vrednost test vektora se cita iz fajla koji je prethodno generisan u MATLAB 
    --referentnoj simulaciji
    stim_proc: PROCESS

    VARIABLE v_line : LINE;
    VARIABLE v_file_name  : STRING(1 TO 21);
    VARIABLE v_data_int: INTEGER;
    VARIABLE v_data : STD_LOGIC_VECTOR(7 DOWNTO 0);
    FILE in_file : TEXT;
    VARIABLE count: STD_LOGIC_VECTOR(10 DOWNTO 0) := (OTHERS => '0');
    VARIABLE ip_frame_length: STD_LOGIC_VECTOR(15 DOWNTO 0) := (OTHERS => '0');

    BEGIN

        --na pocetku simulacije start i valid su neaktivni(logicka nula)
        start_in <= '0';
        data_vld_in <= '0';

        -- cekamo neko vreme
        WAIT FOR 1000 ns;

        --definisemo naziv ulaznog fajla generisan MATLAB referentnom simulacijom.
        --Ovaj fajl sadrzi ip pakete upisane jedan za drugim.
        v_file_name := "../IO_FILES/input.txt";
        --otvaranje reference na fajl
        file_open(in_file, v_file_name, READ_MODE);

        
        --vrtimo se u petlji dok ne procitamo ceo fajl
        WHILE NOT ENDFILE(in_file) LOOP
        --cekamo ulaznu ivicu clock-a
            WAIT UNTIL (clk_125'EVENT and clk_125 = '1');
                --provera se da li je modul spreman da prihvati podatke
                IF(ready_to_accept = '1') THEN
                    --count varijabla broji bajtove koji se citaju iz ulaznog fajla. Kada se procita ceo jedan
                    --ip packet njegova vrednost se resetuje na nulu. Informacije o broju bajtova ip paketa se dobijaju
                    --na osnovu 3 i 4 bajta koji definisu ukupnu duzinu ip paketa.
                    
                    
                    
                    --ip_frame_length cuva informaciju o duzini ip poruke koja se trenutno salje
                    IF(count = conv_std_logic_vector(3,11)) THEN
                        ip_frame_length(15 DOWNTO 8) := data_in;
                    ELSIF(count = conv_std_logic_vector(4,11)) THEN
                        ip_frame_length(7 DOWNTO 0) := data_in;
                    END IF;
                        
                        
                    --Na pocetku svake poruke start signal prelazi u aktivno stanje.
                    IF(count = conv_std_logic_vector(0,11)) THEN
                        start_in <= '1';
                    ELSE
                        start_in <= '0';
                    END IF;
                    
                    --Citamo bajt iz fajla    
                    READLINE(in_file, v_line);
                    READ(v_line,v_data_int); 
                    --Bajt iz fajla prosledjujemo na magistralu                    
                    data_in <= conv_std_logic_vector(v_data_int, 8);
                    --Generisemo valid 
                    data_vld_in <= '1';
                       
                    --Uvecavamo counter za jedan
                    count := count + conv_std_logic_vector(1,11);
                    
                    --Ceo ip paket je poslat resetujemo counter.
                    IF(count = ip_frame_length(10 DOWNTO 0)) THEN
                        count := (OTHERS => '0');                       
                    END IF;

                ELSE --modul nije spreman da prihvati podatke stoga valid i start postaju neaktivni
                    data_vld_in <= '0';
                    start_in <= '0';
                END IF;
            --Kod ispod je zakomentarisan.Moze se odkomentarisati ukoliko se zeli testirati da podaci ne stizu takt za taktom nego izmedju svaka dva valida je jedan takt pauze.
            --WAIT UNTIL (clk_125'EVENT and clk_125 = '1');
            --    start_in <= '0';
            --    data_vld_in <= '0';

        END LOOP;

        --Kada se procita ceo fajl stavlja se valid na neaktivno stanje
        WAIT UNTIL (clk_125'EVENT and clk_125 = '1');
            data_vld_in <= '0';

        --zatvara se referenca na fajl
        file_close(in_file);
        WAIT;
    END PROCESS;
    
    --Ovaj proces sluzi za prihvatanje obradjenih podataka i njihovo cuvanje u izlaznom fajlu
    receive_data: PROCESS

    VARIABLE v_line : LINE;
    VARIABLE v_file_name  : STRING(1 TO 31);
    VARIABLE v_data : std_logic_vector(7 DOWNTO 0);
    FILE out_file : text;

    BEGIN
       -- cekamo neko vreme
       WAIT FOR 500 ns;
       --Naziv izlaznog fajla
       v_file_name := "../IO_FILES/output_modelsim.txt";
       --Otvaranje reference na izlazni fajl
       file_open(out_file, v_file_name, WRITE_MODE);

       --Cekamo na podatke u beskonacnoj petlji
       WHILE (true) LOOP
           WAIT UNTIL (clk_125'EVENT and clk_125 = '1');
            --ukoliko je enable signal aktivan sacuvajmo obradjeni podatak u fajl
            IF (output_enable = '1') THEN
                v_data := data_out;
                WRITE(v_line, conv_integer(v_data));
                WRITELINE(out_file, v_line); 
            END IF;
       END LOOP;

       --zatvaranje reference na fajl
       file_close(out_file);
   END PROCESS receive_data;
    
END tb_ip_frame_signature_a;

